﻿///////////////////////////////////////////////////////////////////////////////
// file levelManager.js
// Copyright (c) 2012 Frédéric J. Rézeau. All rights reserved.
///////////////////////////////////////////////////////////////////////////////

(function (namespace, undefined) {
    "use strict";

    // Level Manager.
    namespace.levelManager = function (level) {
        if (!(this instanceof namespace.levelManager)) {
            throw new Error("Constructor called as a function.");
        }

        // Current level.
        this.currentLevel = level;
        this.currentLevelStage = namespace.LevelStageEnum.None;
        this.currentLevelTip = namespace.LevelTipEnum.None;

        // Total running time for current level in seconds.
        this.runningTime = 0;

        this.refillMessageTime = 0;

        // AI update frequency in seconds.
        this.updateFrequency = 0;


        this.lastSecondUpdate = false;

        // Level dimensions.
        this.minX = 0;
        this.maxX = 0;
        this.maxY = 0;

        // Players.
        this.leftPlayer = {};
        this.rightPlayer = {};

        this.boosters = [{ "activated": false, "free": false }, { "activated": false, "free": false }, { "activated": false, "free": false }, { "activated": false, "free": false }];

        // Collections.
        this.entities = [];             // Entities in the current scene.
        this.effects = [];              // Effects in the current scene.
        this.backgroundEffects = [];    // Background effects in the current scene.
        this.stacks = [];               // Stack in the current scene.
        this.maxStackSize = 3;          // Max stack size per character.

        // New items, zombies, weapon selection related properties.
        this.newItem = namespace.StackObjectTypeEnum.None;
        this.rewardItem = { "crate": namespace.StackObjectTypeEnum.None, "weapon": namespace.WeaponTypeEnum.None, "won": false };
        this.primaryWeaponAssignmentDone = false;
        this.secondaryWeaponAssignmentDone = false;
        this.selectedWeaponType = namespace.WeaponTypeEnum.None;

        this.levelStartCooldown = 0;
        this.levelEndCooldown = 0;
        this.levelEndPauseCooldown = 0;
        this.tutorialCooldown = 0;

        this.killCount = 0;     // Total of kills for the current level.
        this.daysCount = 0;     // Total of days for the current level.

        this.survivalRageTime = 0;
        this.survivalPointsZoomTime = 0;
        this.maxSurvivalRageTime = 7;
        this.survivalPoints = 0;
        this.minSurvivalPoints = 50;

        this.checkpointFrequency = 20;
        this.actualDayDuration = 30;

        this.criticalChance = 0;
        this.repairKits = 0;
        this.showHighScoreEffect = true;

        this.mountedCrateCount = 0;

        this.showBoosters = false;
        this.showProducts = false;
        this.productTime = 0;
        this.showReminder = false;
        this.showReminderTime = 0;
        this.reachedTop = false;

        this.noScratch = true;
        this.survivalRage = false;
        this.usedSpiked = false;
        this.usedTurret = false;
        this.metalCount = 0;
        this.TNTCount = 0;
        this.initialize();
    };

    // Initialize level-based properties.
    namespace.levelManager.prototype.initialize = function () {
        var positionX;

        // Set the level dimensions.
        this.minX = 5;
        this.maxX = 21;
        this.maxY = 5;

        // Compute the base position.
        positionX = this.minX + (this.maxX - this.minX) / 2;

        switch (this.currentLevel) {
            case 1:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.SpikeCrate, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.currentLevelStage = namespace.LevelStageEnum.TutorialStart;
                this.primaryWeaponAssignmentDone = true;
                this.secondaryWeaponAssignmentDone = true;
                this.levelStartCooldown = 0;
                // For the tutorial level 1 we do not want the female player. Lets put her off-screen.
                this.createPlayers(positionX - 0.6, positionX + 100);
                break;
            case 2:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.None, "weapon": namespace.WeaponTypeEnum.ShotGun, "won": false };
                this.primaryWeaponAssignmentDone = true;
                this.secondaryWeaponAssignmentDone = true;
                this.levelStartCooldown = 0;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                break;
            case 3:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.CriticalChance, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = true;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                break;
            case 4:
                this.boosters = [{ "activated": false, "free": true }, { "activated": false, "free": false }, { "activated": false, "free": false }, { "activated": false, "free": false }];
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.None, "weapon": namespace.WeaponTypeEnum.AutoRifle, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = true;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                this.showBoosters = true;
                break;
            case 5:
                this.boosters = [{ "activated": false, "free": false }, { "activated": false, "free": true }, { "activated": false, "free": false }, { "activated": false, "free": false }];
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.MetalCrate, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                this.showBoosters = true;
                break;
            case 6:
                this.boosters = [{ "activated": false, "free": false }, { "activated": false, "free": false }, { "activated": false, "free": true }, { "activated": false, "free": false }];
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.Challenge1, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                this.showBoosters = true;
                break;
            case 7:
                this.boosters = [{ "activated": false, "free": false }, { "activated": false, "free": false }, { "activated": false, "free": false }, { "activated": false, "free": true }];
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.None, "weapon": namespace.WeaponTypeEnum.AutoShotGun, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                this.showBoosters = true;
                break;
            case 8:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.None, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                this.showBoosters = true;
                break;
            case 9:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.None, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                this.showBoosters = true;
                break;
            case 10:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.None, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                this.showBoosters = true;
                break;
            case 11:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.DoubleWoodCrate, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                this.showBoosters = true;
                break;
            case 12:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.None, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                this.showBoosters = true;
                break;
            case 13:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.Challenge2, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                this.showBoosters = true;
                break;
            case 14:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.None, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                this.showBoosters = true;
                break;
            case 15:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.DoubleMetalCrate, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                this.showBoosters = true;
                break;
            case 16:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.None, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                break;
            case 17:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.None, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                this.showBoosters = true;
                break;
            case 18:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.None, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                break;
            case 19:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.None, "weapon": namespace.WeaponTypeEnum.Rpg, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                this.showBoosters = true;
                break;
            case 20:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.None, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.criticalChance = 0.025;
                break;
            case 21:
            case 22:
            case 23:
            case 24:
            case 25:
            case 1000:
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 0;
                this.criticalChance = 0.025;
                this.survivalPoints = 30;
                this.showBoosters = true;
                break;
            case 1001:
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 0;
                this.criticalChance = 0.025;
                this.survivalPoints = 30;
                this.showBoosters = true;
                break;
            default:
                this.rewardItem = { "crate": namespace.StackObjectTypeEnum.DoubleWoodCrate, "weapon": namespace.WeaponTypeEnum.None, "won": false };
                //this.newItem = namespace.StackObjectTypeEnum.TNTCrate;
                this.primaryWeaponAssignmentDone = false;
                this.secondaryWeaponAssignmentDone = false;
                this.levelStartCooldown = 0.7;
                this.updateFrequency = 5;
                this.createPlayers(positionX - 0.6, positionX + 0.6);
                this.survivalPoints = 30;
                this.showBoosters = true;
                break;
        }
    };

    // Create the players.
    namespace.levelManager.prototype.createPlayers = function (x1, x2) {
        // Create the male player.
        this.leftPlayer = new namespace.HumanEntity(x1, this.maxY, 0.4);
        this.leftPlayer.direction = namespace.DirectionEnum.Left;
        this.leftPlayer.buildDirection = namespace.DirectionEnum.Left;
        this.leftPlayer.speed = 0;
        this.leftPlayer.baseSpeed = 0;
        this.leftPlayer.classType = namespace.EntityClassTypeEnum.HumanMale;
        this.leftPlayer.heartBeats[1].maxTime = 4 + Math.random() * 2;
        this.leftPlayer.weapons[namespace.WeaponCategoryEnum.Primary] = namespace.WeaponTypeEnum.Rifle;
        this.leftPlayer.weapons[namespace.WeaponCategoryEnum.Secondary] = namespace.WeaponTypeEnum.Uzi;
        this.leftPlayer.reloadCooldown = 0.5;

        // Create the female player.
        this.rightPlayer = new namespace.HumanEntity(x2, this.maxY, 0.4);
        this.rightPlayer.direction = namespace.DirectionEnum.Right;
        this.rightPlayer.buildDirection = namespace.DirectionEnum.Right;
        this.rightPlayer.speed = 0;
        this.rightPlayer.baseSpeed = 0;
        this.rightPlayer.classType = namespace.EntityClassTypeEnum.HumanFemale;
        this.rightPlayer.heartBeats[1].maxTime = 4 + Math.random() * 2;
        this.rightPlayer.weapons[namespace.WeaponCategoryEnum.Primary] = namespace.WeaponTypeEnum.Rifle;
        this.rightPlayer.weapons[namespace.WeaponCategoryEnum.Secondary] = namespace.WeaponTypeEnum.Uzi;
        this.rightPlayer.reloadCooldown = 0.5;

        // Create the player stacks.
        this.leftPlayer.stack = { "x": this.leftPlayer.x, "vx": 0, "objects": [], "limitX": 0, "direction": namespace.DirectionEnum.None, "cooldown": 0 };
        this.rightPlayer.stack = { "x": this.rightPlayer.x, "vx": 0, "objects": [], "limitX": 0, "direction": namespace.DirectionEnum.None, "cooldown": 0 };
        this.stacks.push(this.leftPlayer.stack);
        this.stacks.push(this.rightPlayer.stack);

        if (this.currentLevel !== 1) {
            var stack = { "x": this.leftPlayer.x + ((this.rightPlayer.x - this.leftPlayer.x) / 2), "vx": 0, "objects": [], "limitX": 0, "direction": namespace.DirectionEnum.None, "cooldown": 0 };
            stack.objects.push(new namespace.StackObject(this.maxY, 0.6, namespace.StackObjectTypeEnum.WoodCrate, 100));
            this.stacks.push(stack);
        }
    };

    // Update time-based properties.
    namespace.levelManager.prototype.update = function (elapsed) {
        this.runningTime += elapsed;

        if (this.currentLevel === 1001) {
            this.runningTime = 0;
        }

        if (this.showReminderTime > 0) {
            this.showReminderTime -= elapsed;
            if (this.showReminderTime <= 0) {
                this.showReminderTime = 0;
                this.showReminder = false;
            }
        }

        // Survival rage time.
        if (this.survivalRageTime > 0) {
            this.survivalRageTime -= elapsed;
            if (this.survivalRageTime < 0) {
                this.survivalRageTime = 0;
            }
        }

        if (this.productTime > 0) {
            this.productTime -= elapsed;
            if (this.productTime < 0) {
                this.productTime = 0;
            }
        }

        if (this.refillMessageTime > 0) {
            this.refillMessageTime -= elapsed;
            if (this.refillMessageTime < 0) {
                this.refillMessageTime = 0;
            }
        }

        if (this.survivalPointsZoomTime > 0) {
            this.survivalPointsZoomTime -= elapsed;
            if (this.survivalPointsZoomTime < 0) {
                this.survivalPointsZoomTime = 0;
            }
        }

        if (this.levelEndCooldown > 0) {
            this.levelEndCooldown -= elapsed;
            if (this.levelEndCooldown < 0) {
                this.levelEndCooldown = 0;
            }
        }

        if (this.levelEndPauseCooldown > 0) {
            this.levelEndPauseCooldown -= elapsed;
            if (this.levelEndPauseCooldown < 0) {
                this.levelEndPauseCooldown = 0;
            }
        }

        if (this.primaryWeaponAssignmentDone && this.secondaryWeaponAssignmentDone && this.newItem === namespace.StackObjectTypeEnum.None) {
            if (this.levelStartCooldown > 0) {
                this.levelStartCooldown -= elapsed;
                if (this.levelStartCooldown < 0) {
                    this.levelStartCooldown = 0;
                }
            }
            else {
                if (this.tutorialCooldown > 0) {
                    this.tutorialCooldown -= elapsed;
                    if (this.tutorialCooldown < 0) {
                        this.tutorialCooldown = 0;
                    }
                }
                if (this.currentLevelStage === namespace.LevelStageEnum.TutorialZombiesComing) {
                    if (this.tutorialCooldown === 0) {
                        this.currentLevelStage = namespace.LevelStageEnum.TutorialHitBuildButton;
                    }
                }
                else if (this.currentLevelStage === namespace.LevelStageEnum.TutorialMoreZombiesComing) {
                    if (this.tutorialCooldown === 0) {
                        this.currentLevelStage = namespace.LevelStageEnum.TutorialTurnRight;
                    }
                }
            }
        }
    };

    // Get the stack object creation time.
    namespace.levelManager.prototype.getStackObjectCreationTime = function (type) {
        switch (type) {
            case namespace.StackObjectTypeEnum.WoodCrate:
                return 3;
            case namespace.StackObjectTypeEnum.MetalCrate:
                return 5;
            case namespace.StackObjectTypeEnum.SpikeCrate:
                return 3;
            case namespace.StackObjectTypeEnum.TNTCrate:
                return 6;
            case namespace.StackObjectTypeEnum.MountedCrate:
                return 8;
            case namespace.StackObjectTypeEnum.RepairCrate:
                return 1;
            case namespace.StackObjectTypeEnum.DoubleMetalCrate:
                return 3;
            case namespace.StackObjectTypeEnum.DoubleWoodCrate:
                return 5;
        }
        return 0;
    };

    // Get the stack object strength.
    namespace.levelManager.prototype.getStackObjectStrength = function (stackObjectType) {
        switch (stackObjectType) {
            case namespace.StackObjectTypeEnum.WoodCrate:
                return 3;
            case namespace.StackObjectTypeEnum.TNTCrate:
                return 5;
            case namespace.StackObjectTypeEnum.SpikeCrate:
                return 5;
            case namespace.StackObjectTypeEnum.MetalCrate:
                return 8;
            case namespace.StackObjectTypeEnum.MountedCrate:
                return 10;
        }
        return 1;
    };

})(window.ZCJGame = window.ZCJGame || {});
