﻿///////////////////////////////////////////////////////////////////////////////
// file main.js
// Copyright (c) 2012 Frédéric J. Rézeau. All rights reserved.
///////////////////////////////////////////////////////////////////////////////

// Provides requestAnimationFrame in a cross browser way. @author paulirish / http://paulirish.com/
if (!window.requestAnimationFrame) {
    window.requestAnimationFrame = (function () {
        "use strict";
        return window.webkitRequestAnimationFrame || window.mozRequestAnimationFrame || window.oRequestAnimationFrame || window.msRequestAnimationFrame ||
        function (callback) {
            window.setTimeout(callback, 1000 / 60);
        };
    })();
}

window.onload = (function () {
    "use strict";
    if (typeof CocoonJS !== "undefined") {
        CocoonJS.App.setAppShouldFinishCallback(onNavigateBack);

        // Store initialization.
        CocoonJS.Store.requestInitialization({ sandbox: false, managed: true });
        CocoonJS.Store.start();
    }
    return ZCJGame.initialize();
})();

// Window is resized.
function resize () {
    "use strict";

    // Reset the display.
    ZCJGame.resetDisplay();
};




///////////////////////////////////////////////////////////////////////////////
// file main.js
// Copyright (c) 2012 Frédéric J. Rézeau. All rights reserved.
///////////////////////////////////////////////////////////////////////////////

window.onload = (function () {
    "use strict";

    var main = function () {

        // Configure mopub ads.
        if (typeof Cocoon !== "undefined" && Cocoon.Ad && Cocoon.Ad.MoPub) {
            Cocoon.Ad.MoPub.configure({
                ios: {
                    interstitial: "a13cd765323042db8699f054d9cccf94",
                },
                android: {
                    interstitial: "fa4a0bfb02d74b47ac3f411f59949c2d"
                }
            });
        }

        // Configure IAP.
        if (typeof Cocoon !== "undefined" && Cocoon.InApp) {
            Cocoon.InApp.on("purchase", {
                start: function (productId) {
                    console.log("purchase started " + productId);
                },
                error: function (productId, error) {
                    console.log("purchase failed " + productId + " error: " + JSON.stringify(error));
                },
                complete: function (purchase) {
                    console.log("purchase completed " + JSON.stringify(purchase));

                    switch (purchase.productId) {
                        case "001":
                            window.ZCJGame.addGold(1000);
                            break;
                        case "002":
                            window.ZCJGame.addGold(1500);
                            break;
                        case "003":
                            window.ZCJGame.addGold(2000);
                            break;
                        case "004":
                            window.ZCJGame.addGold(3000);
                            break;
                        case "005":
                            window.ZCJGame.addGold(5000);
                            break;
                        case "006":
                            window.ZCJGame.addGold(10000);
                            break;
                    }

                    Cocoon.InApp.consume(purchase.productId, 1, function (error) { });
                }
            });

            // Service initialization
            Cocoon.InApp.initialize({}, function (error) { });
        }

        // Back button handler.
        document.addEventListener("backbutton", onNavigateBack, false);

        // Orientation changed event.
        window.addEventListener("orientationchange",
            function () {
                ZCJGame.resetDisplay();
            }, false);

        // Initialize the game.
        ZCJGame.initialize();
    };

    document.addEventListener("deviceready", main, false);

    if (window.TestLocalRun === "afygr") {
        main();
    }

})();




