﻿///////////////////////////////////////////////////////////////////////////////
// file soundmanager.js
// Copyright (c) 2012 Frédéric J. Rézeau. All rights reserved.
///////////////////////////////////////////////////////////////////////////////

(function (namespace, undefined) {
    "use strict";

    var gunCooldown = 0;
    var machineGunCooldown = 0;
    var machineGunCooldown2 = 0;
    var humanCoodown = 0;
    var critCooldown = 0;

    var levelMusic = null;
    var mainMusic = null;
    var soundEnabled = 2;

    var soundsMap = {
        "critical": "assets/sound/crit.ogg",
        "rifle": "assets/sound/rifle.ogg",
        "zombieroar1": "assets/sound/zombie1.ogg",
        "zombieroar2": "assets/sound/zombie2.ogg",
        "humandie": "assets/sound/die3.ogg",
        "humansuccess1": "assets/sound/oi.ogg",
        "humansuccess2": "assets/sound/yippee.ogg",
        "humanfail": "assets/sound/boing.ogg",
        "victory": "assets/sound/victory.ogg",
        "award": "assets/sound/award.ogg",
        "zombieattack1": "assets/sound/attack.ogg",
        "zombieattack2": "assets/sound/attack2.ogg",
        "zombiedie1": "assets/sound/die.ogg",
        "zombiedie2": "assets/sound/die2.ogg",
        "reload": "assets/sound/reload.ogg",
        "hammer": "assets/sound/hammer.ogg",
        "autogun1": "assets/sound/uzi.ogg",
        "autogun2": "assets/sound/uzi.ogg",
        "highscore": "assets/sound/score.ogg",
        "checkpoint": "assets/sound/checkpoint.ogg",
        "click": "assets/sound/button.ogg",
        "car": "assets/sound/car.ogg",
        "blast": "assets/sound/launcher.ogg"
    };

    var hasExternalNotify = (typeof window.external !== "undefined" && typeof window.external.notify !== "undefined");

    // Preload images with completion callback.
    var sounds = {};
    namespace.initialize = function (callback) {
        var i, len = 0, loadedSounds = 0;
        var onLoad = function () {
            if (++loadedSounds === len + 2 && callback) {
                callback();
            }
        };

        for (i in soundsMap) {
            if (soundsMap.hasOwnProperty(i)) {
                len++;
            }
        }

        for (i in soundsMap) {
            if (soundsMap.hasOwnProperty(i)) {
                sounds[i] = new Audio();
                sounds[i].addEventListener("canplaythrough", onLoad);
                sounds[i].src = soundsMap[i];
                sounds[i].load();
            }
        }

        levelMusic = new Audio();
        levelMusic.addEventListener("canplaythrough", onLoad);
        levelMusic.src = "assets/sound/music1.ogg";
        levelMusic.load();

        mainMusic = new Audio();
        mainMusic.addEventListener("canplaythrough", onLoad);
        mainMusic.src = "assets/sound/music3.ogg";
        mainMusic.load();
    };

    namespace.update = function (elapsed) {
        if (gunCooldown > 0) {
            gunCooldown -= elapsed;
            if (gunCooldown < 0) {
                gunCooldown = 0;
            }
        }

        if (machineGunCooldown > 0) {
            machineGunCooldown -= elapsed;
            if (machineGunCooldown < 0) {
                machineGunCooldown = 0;
            }
        }

        if (machineGunCooldown2 > 0) {
            machineGunCooldown2 -= elapsed;
            if (machineGunCooldown2 < 0) {
                machineGunCooldown2 = 0;
            }
        }

        if (humanCoodown > 0) {
            humanCoodown -= elapsed;
            if (humanCoodown < 0) {
                humanCoodown = 0;
            }
        }

        if (critCooldown > 0) {
            critCooldown -= elapsed;
            if (critCooldown < 0) {
                critCooldown = 0;
            }
        }

    };

    namespace.getSoundsEnabled = function () {
        return soundEnabled;
    };

    namespace.playSound = function (id) {
        if (hasExternalNotify) {
            window.external.notify(id);
        }
        else {
            var sound = sounds[id];
            sound.currentTime = 0;
            sound.play();
        }
    };

    namespace.playMainMusic = function () {
        if (soundEnabled > 1) {
            if (hasExternalNotify) {
                window.external.notify("commandStopLevelMusic");
                window.external.notify("commandStopMainMusic");
                window.external.notify("commandPlayMainMusic");
            }
            else {
                if (levelMusic) {
                    levelMusic.pause();
                }

                if (mainMusic) {
                    mainMusic.currentTime = 0;
                    mainMusic.play();
                }
            }
        }
    };

    namespace.playLevelMusic = function () {
        if (soundEnabled > 1) {
            if (hasExternalNotify) {
                window.external.notify("commandStopMainMusic");
                window.external.notify("commandStopLevelMusic");
                window.external.notify("commandPlayLevelMusic");
            }
            else {
                if (mainMusic) {
                    mainMusic.pause();
                }

                if (levelMusic) {
                    setTimeout(function () {
                        levelMusic.currentTime = 0;
                        levelMusic.play();
                    }, 300);
                }
            }
        }
    };

    namespace.playCritSound = function () {
        if (soundEnabled > 0 && critCooldown === 0) {
            critCooldown = 3;
            namespace.playSound("critical");
        }
    };

    namespace.playCheckpointSound = function () {
        if (soundEnabled > 0) {
            namespace.playSound("checkpoint");
        }
    };

    namespace.playHighScoreSound = function () {
        if (soundEnabled > 0) {
            namespace.playSound("highscore");
        }
    };

    namespace.playGunSound = function () {
        if (soundEnabled > 0 && gunCooldown === 0) {
            gunCooldown = 0.1;
            namespace.playSound("rifle");
        }
    };

    namespace.playZombieComing = function () {
        if (soundEnabled > 0) {
            namespace.playSound(((Math.random() > 0.5) ? "zombieroar1" : "zombieroar2"));
        }
    };

    namespace.playHumanDie = function () {
        if (soundEnabled > 0) {
            namespace.playSound("humandie");
        }
    };

    namespace.playHumanSuccess = function () {
        if (soundEnabled > 0) {
            if (soundEnabled && humanCoodown === 0) {
                humanCoodown = 2;
                namespace.playSound(((Math.random() > 0.5) ? "humansuccess1" : "humansuccess2"));
            }
        }
    };

    namespace.playHumanFailed = function () {
        if (soundEnabled > 0) {
            namespace.playSound("humanfail");
        }
    };

    namespace.playVictory = function () {
        if (soundEnabled > 0) {
            if (hasExternalNotify) {
                window.external.notify("commandStopLevelMusic");
            }
            else {
                if (levelMusic) {
                    levelMusic.pause();
                }
            }
            namespace.playSound("victory");
        }
    };

    namespace.playAward = function () {
        if (soundEnabled > 0) {
            namespace.playSound("award");
        }
    };

    namespace.playZombieAttackSound = function () {
        if (soundEnabled > 0) {
            namespace.playSound(((Math.random() > 0.5) ? "zombieattack1" : "zombieattack2"));
        }
    };

    namespace.playZombieDieSound = function () {
        if (soundEnabled > 0) {
            namespace.playSound(((Math.random() > 0.5) ? "zombiedie1" : "zombiedie2"));
        }
    };

    namespace.playReloadSound = function () {
        if (soundEnabled > 0) {
            namespace.playSound("reload");
        }
    };

    namespace.playCarSound = function () {
        if (soundEnabled > 0) {
            namespace.playSound("car");
        }
    };

    namespace.playBlastSound = function () {
        if (soundEnabled > 0) {
            namespace.playSound("blast");
        }
    };

    namespace.playClickSound = function () {
        if (soundEnabled > 0) {
            namespace.playSound("click");
        }
    };

    namespace.playHammerSound = function () {
        if (soundEnabled > 0) {
            namespace.playSound("hammer");
        }
    };

    namespace.playMachineGunSound = function () {
        if (soundEnabled > 0 && machineGunCooldown === 0) {
            machineGunCooldown = 0.5;
            namespace.playSound("autogun1");
        }
    };

    namespace.playMachineGunSound2 = function () {
        if (soundEnabled > 0 && machineGunCooldown2 === 0) {
            machineGunCooldown2 = 0.6;
            namespace.playSound("autogun2");
        }
    };

    namespace.mute = function (all) {
        if (hasExternalNotify) {
            window.external.notify("commandPauseMainMusic");
            window.external.notify("commandPauseLevelMusic");
        }
        else {
            if (mainMusic) {
                mainMusic.pause();
            }

            if (levelMusic) {
                levelMusic.pause();
            }
        }
        soundEnabled = (all === true) ? 0 : 1;
    };

    namespace.unmute = function (all) {
        if (all === true) {
            if (hasExternalNotify) {
                window.external.notify("commandPlayMainMusic");
            }
            else {

                if (mainMusic) {
                    mainMusic.play();
                }
            }
        }
        soundEnabled = (all === true) ? 2 : 1;
    };

})(window.ZCJGame.SoundManager = window.ZCJGame.SoundManager || {});
