﻿///////////////////////////////////////////////////////////////////////////////
// file stacks.js
// Copyright (c) 2012 Frédéric J. Rézeau. All rights reserved.
///////////////////////////////////////////////////////////////////////////////

(function (namespace, undefined) {
    "use strict";

    // Stack Object.
    namespace.StackObject = function (y, radius, type, strength) {
        if (!(this instanceof namespace.StackObject)) {
            throw new Error("Constructor called as a function.");
        }

        this.y = y;                     // Position y.
        this.vy = 0;                    // Velocity y.
        this.radius = radius;           // Bounding circle radius.
        this.rotationSpeed = 0;         // Rotation speed.
        this.angle = 0;                 // Current angle.
        this.gravity = 9.78;            // Gravity factor.
        this.type = type;               // Object type.
        this.strength = strength;       // Strength.
        this.firingCooldown = 0;        // Firing cooldown.
        this.firingAngle = 0;           // Firing angle.
        this.firingRecoilTime = 0;          // Firing recoil time.
        this.direction = namespace.DirectionEnum.None;
        this.selected = false;
        this.visible = true;
    };

    // Update the stack object time-based properties.
    namespace.StackObject.prototype.update = function (elapsed) {
        // Update the firing cooldown.
        if (this.firingCooldown > 0) {
            this.firingCooldown -= elapsed;
            if (this.firingCooldown < 0) {
                this.firingCooldown = 0;
            }
        }

        // Update the firing recoil time.
        if (this.firingRecoilTime > 0) {
            this.firingRecoilTime -= elapsed;
            if (this.firingRecoilTime < 0) {
                this.firingRecoilTime = 0;
            }
        }

        // Apply friction on rotation speed.
        if (this.rotationSpeed > 0) {
            this.rotationSpeed -= 2 * elapsed;
            if (this.rotationSpeed < 0) {
                this.rotationSpeed = 0;
            }
        }
        else if (this.rotationSpeed < 0) {
            this.rotationSpeed += 2 * elapsed;
            if (this.rotationSpeed > 0) {
                this.rotationSpeed = 0;
            }
        }

        // Restore the object angle.
        if (this.rotationSpeed === 0 && this.angle !== 0) {
            if (this.angle > 0) {
                this.angle -= (Math.PI * 2) * elapsed;
                if (this.angle < 0) {
                    this.angle = 0;
                }
            }
            else {
                this.angle += (Math.PI * 2) * elapsed;
                if (this.angle > 0) {
                    this.angle = 0;
                }
            }
        }

        // Apply rotation to angle.
        this.angle = this.rotationSpeed ? (this.angle + this.rotationSpeed * (Math.PI * 2) * elapsed) % (Math.PI * 2) : this.angle;

        // Apply gravity.
        this.vy += this.gravity * elapsed;

        // Apply velocity.
        this.y += this.vy * elapsed;
    };

})(window.ZCJGame = window.ZCJGame || {});
